from datetime import datetime

import pymongo
from discord.ext import commands
from discord_slash import SlashContext, cog_ext
from discord_slash.utils.manage_commands import create_option

from jarvis.config import get_config
from jarvis.db import DBManager
from jarvis.db.types import Lock
from jarvis.utils.cachecog import CacheCog


class LockdownCog(CacheCog):
    def __init__(self, bot: commands.Bot):
        super().__init__(bot)
        self.db = DBManager(get_config().mongo).mongo.jarvis

    @cog_ext.cog_subcommand(
        base="lockdown",
        name="start",
        description="Locks a server",
        options=[
            create_option(
                name="reason",
                description="Lockdown Reason",
                option_type=3,
                required=True,
            ),
            create_option(
                name="duration",
                description="Lockdown duration in minutes (default 10)",
                option_type=4,
                required=False,
            ),
        ],
    )
    @commands.has_permissions(administrator=True)
    async def _lockdown_start(
        self,
        ctx: SlashContext,
        reason: str,
        duration: int = 10,
    ):
        await ctx.defer(hidden=True)
        if duration <= 0:
            await ctx.send("Duration must be > 0", hidden=True)
            return
        elif duration >= 300:
            await ctx.send("Duration must be < 5 hours", hidden=True)
            return
        channels = ctx.guild.channels
        roles = ctx.guild.roles
        updates = []
        for channel in channels:
            for role in roles:
                try:
                    await self._lock_channel(channel, role, ctx.author, reason)
                except Exception:
                    continue  # Just continue on error
            updates.append(
                pymongo.InsertOne(
                    {
                        "channel": channel.id,
                        "guild": ctx.guild.id,
                        "admin": ctx.author.id,
                        "reason": reason,
                        "duration": duration,
                        "active": True,
                        "created_at": datetime.utcnow(),
                    }
                )
            )
        if updates:
            self.db.locks.bulk_write(updates)
        await ctx.send(f"Server locked for {duration} minute(s)")

    @cog_ext.cog_subcommand(
        base="lockdown",
        name="end",
        description="Unlocks a server",
    )
    @commands.has_permissions(administrator=True)
    async def _lockdown_end(
        self,
        ctx: SlashContext,
    ):
        channels = ctx.guild.channels
        roles = ctx.guild.roles
        updates = []
        locks = Lock.get_many(guild=ctx.guild.id, active=True)
        if not locks:
            await ctx.send("No lockdown detected.", hidden=True)
            return
        await ctx.defer()
        for channel in channels:
            for role in roles:
                try:
                    await self._unlock_channel(channel, role, ctx.author)
                except Exception:
                    continue  # Just continue on error
            updates.append(
                pymongo.UpdateOne(
                    {
                        "channel": channel.id,
                        "guild": ctx.guild.id,
                        "admin": ctx.author.id,
                    },
                    {"$set": {"active": False}},
                )
            )
        if updates:
            self.db.locks.bulk_write(updates)
        await ctx.send("Server unlocked")
